<?php

namespace Bookit\Classes;

use Bookit\Classes\Admin\SettingsController;
use Bookit\Classes\Database\Appointments;
use Bookit\Classes\Database\Customers;
use Bookit\Classes\Database\Services;
use Bookit\Classes\Database\Staff;

class Notifications {

	/**
	 * Init Notifications
	 */
	public static function init() {
		add_action('bookit_appointment_created', [self::class, 'appointment_created'], 100, 1);
		add_action('bookit_appointment_updated', [self::class, 'appointment_updated'], 100, 1);
		add_action('bookit_payment_complete', [self::class, 'payment_complete'], 100, 1);
		add_action('bookit_appointment_status_changed', [self::class, 'appointment_status_changed'], 100, 1);
		add_action('bookit_appointment_deleted', [self::class, 'appointment_deleted'], 100, 3);
		add_filter('bookit_filter_email_data', [self::class, 'rewrite_email_data'], 10, 1);
	}

	/**
	 * Mail Content Type Filter
	 * @return string
	 */
	public static function mail_content_type() {
		return 'text/plain';
	}

	/**
	 * Sent Email
	 * @param $to
	 * @param $subject
	 * @param $body
	 * @param $vars
	 * @param $template
	 */
	public static function sent_mail( $to, $subject, $body, $vars = [], $template = '' ) {
		$headers[]  = 'Content-Type: text/plain; charset=UTF-8';

		add_filter('wp_mail_content_type', [self::class, 'mail_content_type']);

		$data = apply_filters('bookit_filter_email_data', [
			'to'        => $to,
			'subject'   => $subject,
			'body'      => $body,
			'vars'      => $vars,
			'template'  => $template
		]);

		wp_mail($data['to'], $data['subject'], $data['body'], $headers);

		remove_filter('wp_mail_content_type', [self::class, 'mail_content_type']);
	}

	/**
	 * Appointment Created
	 * @param $appointment_id
	 */
	public static function appointment_created( $appointment_id ) {
		$settings   = SettingsController::get_settings();
		$vars       = self::get_email_variables($appointment_id);

		if ( ! empty( $settings['emails']['appointment_created_customer']['enabled'] ) ) {
			self::sent_mail($vars['[customer]'], '', '', $vars, 'appointment_created_customer');
		}

		if ( ! empty( $settings['emails']['appointment_created_admin']['enabled'] ) ) {
			self::sent_mail($vars['[admin]'], '', '', $vars, 'appointment_created_admin');
		}
	}

	/**
	 * Appointment Updated
	 * @param $appointment_id
	 */
	public static function appointment_updated( $appointment_id ) {
		$settings   = SettingsController::get_settings();
		$vars       = self::get_email_variables($appointment_id);

		if ( ! empty( $settings['emails']['appointment_updated_customer']['enabled'] ) ) {
			self::sent_mail($vars['[customer]'], '', '', $vars, 'appointment_updated_customer');
		}

		if ( ! empty( $settings['emails']['appointment_updated_admin']['enabled'] ) ) {
			self::sent_mail($vars['[admin]'], '', '', $vars, 'appointment_updated_admin');
		}
	}

	/**
	 * Payment Complete Notification
	 * @param $appointment_id
	 */
	public static function payment_complete( $appointment_id ) {
		$settings   = SettingsController::get_settings();
		$vars       = self::get_email_variables($appointment_id);

		if ( ! empty( $settings['emails']['payment_complete_customer']['enabled'] ) ) {
			self::sent_mail($vars['[customer]'], '', '', $vars, 'payment_complete_customer');
		}

		if ( ! empty( $settings['emails']['payment_complete_admin']['enabled'] ) ) {
			self::sent_mail($vars['[admin]'], '', '', $vars, 'payment_complete_admin');
		}
	}

	/**
	 * Appointment Status Changed Notification
	 * @param $appointment_id
	 */
	public static function appointment_status_changed( $appointment_id ) {
		$settings   = SettingsController::get_settings();
		$vars       = self::get_email_variables($appointment_id);

		if ( ! empty( $settings['emails']['appointment_status_changed']['enabled'] ) ) {
			self::sent_mail($vars['[customer]'], '', '', $vars, 'appointment_status_changed');
		}
	}

	/**
	 * Get Email Variables
	 * @param $appointment_id
	 * @return array
	 */
	public static function get_email_variables( $appointment_id, $reason = '' ) {
		$appointment    = Appointments::get('id', $appointment_id);
		$customer       = Customers::get('id', $appointment->customer_id);
		$staff          = Staff::get('id', $appointment->staff_id);
		$service        = Services::get('id', $appointment->service_id);

		$vars = [
			'[admin]'           => get_bloginfo('admin_email'),
			'[staff]'           => $staff->email,
			'[customer]'        => $customer->email,
			'[staff_name]'      => $staff->full_name,
			'[staff_phone]'     => $staff->phone,
			'[customer_name]'   => $customer->full_name,
			'[customer_email]'  => $customer->email,
			'[customer_phone]'  => $customer->phone,
			'[service_title]'   => $service->title,
			'[appointment_id]'  => $appointment->id,
			'[appointment_day]' => date( get_option('date_format'), $appointment->date_timestamp ),
			'[start_time]'      => date( get_option('time_format'), $appointment->start_time ),
			'[payment_method]'  => $appointment->payment_method,
			'[payment_status]'  => $appointment->payment_status,
			'[total]'           => bookit_price($appointment->price),
			'[status]'          => $appointment->status,
			'[reason]'          => $reason,
		];

		return $vars;
	}

	/**
	 * Rewrite Email Data
	 * @param $data
	 * @return mixed
	 */
	public static function rewrite_email_data( $data ) {
		if ( empty( $data['vars'] ) or empty( $data['template'] ) ) {
			return $data;
		}

		$template   = $data['template'];
		$settings   = SettingsController::get_settings();

		if ( ! empty( $settings['emails'][$template]['to'] ) ) {
			$data['to'] = strtr( $settings['emails'][$template]['to'], $data['vars'] );
		}

		if ( ! empty( $settings['emails'][$template]['subject'] ) ) {
			$data['subject'] = strtr( $settings['emails'][$template]['subject'], $data['vars'] );
		}

		if ( ! empty( $settings['emails'][$template]['body'] ) ) {
			$data['body'] = strtr( $settings['emails'][$template]['body'], $data['vars'] );
		}

		return $data;
	}

	/**
	 * Appointment Delete Notification
	 *
	 * @param int $appointment_id
	 * @param string $send_notification_to
	 */
	public static function appointment_deleted(int $appointment_id, array $send_notification_to, $reason ) {

		$vars = self::get_email_variables($appointment_id, $reason);
		if ( $send_notification_to['customer'] ) {
			self::sent_mail($vars['[customer]'], '', '', $vars, 'appointment_deleted_customer');
		}

		if ( $send_notification_to['staff'] ) {
			self::sent_mail($vars['[staff]'], '', '', $vars, 'appointment_deleted_staff');
		}
	}


}