<?php

namespace Bookit\Classes\Admin;

use Bookit\Classes\Database\Categories;
use Bookit\Classes\Database\Services;
use Bookit\Classes\Database\Staff;
use Bookit\Classes\Template;
use Bookit\Classes\Vendor\Payments;
use Bookit\Helpers\CleanHelper;
use Bookit\Helpers\MailTemplateHelper;
use Bookit\Helpers\TimeSlotHelper;

class SettingsController extends DashboardController {

	static $default_currency = 'usd';
	static $settings_key = 'bookit_settings';
	static $time_slot_default_duration = 15;

	private static function getCleanRules() {
		return [
			'booking_type'     => ['type' => 'strval' ],
			'theme'     => ['type' => 'strval' ],
			'hide_header_titles'     => ['type' => 'strval' ],
			'currency_symbol'     => ['type' => 'strval' ],
			'currency_position'     => ['type' => 'strval' ],
			'thousands_separator'     => ['type' => 'strval' ],
			'decimals_separator'     => ['type' => 'strval' ],
			'decimals_number'     => ['type' => 'intval' ],
			'custom_colors_enabled'     => ['type' => 'strval' ],
			'currency'             => ['function' => ['custom' => false, 'name' => 'strtolower']],
			'payments'             => ['function' => ['custom' => true, 'name' => 'custom_sanitize_json']],
			'emails'             => ['function' => ['custom' => true, 'name' => 'custom_sanitize_json']],
			'custom_colors' => ['function' => ['custom' => true, 'name' => 'custom_sanitize_json']],
		];
	}

	/**
	 * Display Rendered Template
	 * @return bool|string
	 */
	public static function render() {
		self::enqueue_styles_scripts();

		return Template::load_template(
			'dashboard/bookit-settings',
			[
				'settings'              => self::get_settings(),
				'pro_disabled'          => bookit_pro_features_disabled(),
				'woocommerce_enabled'   => class_exists('WooCommerce')? 'true' : 'false',
				'woocommerce_products'  => self::bookit_woocommerce_get_all_products(),
				'categories'            => Categories::get_all(),
				'services'              => Services::get_all(),
				'staff'                 => Staff::get_all_shorted(),
				'currencies'            => Payments::get_currency_list(),
				'time_slot_options'     => TimeSlotHelper::TIME_SLOT_POSSIBLE_VALUES,
			],
			true
		);
	}

	/**
	 * Validate data
	 */
	public static function validate( $data ) {
		$errors = [];

		$is_currency = array_search(strtoupper($data['currency']), array_column(Payments::get_currency_list(), 'value'));

		if ( strlen($data['currency']) != 3 || false === $is_currency ) {
			$errors['currency'] = __('Wrong currency value');
		}

		if ( count($errors ) > 0 ) {
			wp_send_json_error( ['errors' => $errors, 'message' => __('Error occurred!', 'bookit')] );
		}
	}

	/**
	 * Save Settings
	 */
	public static function save() {
		check_ajax_referer('bookit_save_settings', 'nonce');

		if ( !current_user_can( 'manage_options' ) ) {
			return false;
		}

		$data = CleanHelper::cleanData( $_POST, self::getCleanRules() );
		self::validate( $data );

		if ( ! empty( $data ) ) {

			unset($data['nonce']);

			do_action('bookit_before_update_setting', $data);

			self::save_settings($data);

			wp_send_json_success( [ 'message' => __( 'Settings Saved!', 'bookit' )] );
		}

		wp_send_json_error( [ 'message' => __('Error occurred!', 'bookit') ] );
	}

	/**
	 * Save Default Settings
	 * @return boolean
	 */
	public static function save_default_settings() {
		if ( empty( (array) self::get_settings() ) ) {
			$settings = [
				'booking_type'          => 'registered',
				'theme'                 => 'default',
				'time_slot_duration'    => static::$time_slot_default_duration,
				'hide_header_titles'    => false,
				'currency'              => static::$default_currency,
				'currency_symbol'       => '$',
				'currency_position'     => 'left',
				'thousands_separator'   => ',',
				'decimals_separator'    => '.',
				'decimals_number'       => 2,
				'custom_colors_enabled' => false,
				'custom_colors'         => [
					'base_color'        => '#006666',
					'base_bg_color'     => '#f0f8f8',
					'highlight_color'   => '#ffd400',
					'white_color'       => '#ffffff',
					'dark_color'        => '#272727',
				],
				'payments'              => [
					'locally'       => [
						'enabled'       => true
					],
					'paypal'        => [
						'enabled'       => false
					],
					'stripe'        => [
						'enabled'       => false
					],
					'woocommerce'   => [
						'enabled'       => false
					],
				],
				'emails'              => MailTemplateHelper::getTemplates(),
			];

			return self::save_settings($settings);
		}

		return true;
	}

	/**
	 * Get Settings
	 * @return mixed
	 */
	public static function get_settings() {
		return get_option( self::$settings_key, (object) [] );
	}

	/**
	 * Save Settings
	 * @param $settings
	 * @return mixed
	 */
	public static function save_settings( $settings ) {
		update_option( self::$settings_key, $settings, self::getCleanRules() );
	}

	/**
	 * Get All Products
	 * @return array
	 */
	public static function bookit_woocommerce_get_all_products() {
		$products       = [];

		if ( bookit_pro_active() && class_exists('WooCommerce')) {
			$args           = array( 'post_type' => 'product', 'posts_per_page' => -1 );
			$all_products   = get_posts( $args );

			foreach ( $all_products as $key => $product ) {
				$products[$key]['id']       = $product->ID;
				$products[$key]['title']    = $product->post_title;
			}
		}

		return $products;
	}

}