<?php

namespace Bookit\Classes\Admin;

use Bookit\Classes\Database\Appointments;
use Bookit\Classes\Database\Services;
use Bookit\Classes\Database\Staff;
use Bookit\Classes\Database\Staff_Services;
use Bookit\Classes\Database\Staff_Working_Hours;
use Bookit\Classes\Template;
use Bookit\Helpers\CleanHelper;

class StaffController extends DashboardController {

	private static function getCleanRules() {
		return [
			'id'            => [ 'type' => 'intval' ],
			'limit'         => [ 'type' => 'intval' ],
			'offset'        => [ 'type' => 'intval' ],
			'email'         => [ 'type' => 'strval' ],
			'full_name'     => ['type' => 'strval' ],
			'phone'         => ['function' => ['custom' => true, 'name' => 'custom_sanitize_phone']],
		];
	}
	/**
	 * Display Rendered Template
	 * @return bool|string
	 */
	public static function render() {
		$staff      = Staff::get_all();
		$services   = Services::get_all_short();

		self::enqueue_styles_scripts();

		return Template::load_template(
			'dashboard/bookit-staff',
			[ 'staff' => $staff, 'services' => $services ],
			true
		);
	}

	/**
	 * Get Staff with Pagination
	 */
	public static function get_staff() {

		$data = CleanHelper::cleanData($_GET, self::getCleanRules());

		if ( ! empty( $data['limit'] ) ) {
			$response['staff'] = Staff::get_paged( $data['limit'], $data['offset'] );
			$response['total'] = Staff::get_count();

			wp_send_json_success( $response );
		}

		wp_send_json_error( [ 'message' => __('Error occurred!', 'bookit') ] );
	}

	/**
	 * Validate staff fields
	 */
	public static function validate( $data ) {
		$errors = [];

		if ( $data['phone'] || $data['phone'] === false ) {
			if ( !preg_match('/^((\+)?[0-9]{9,14})$/', $data['phone']) ) {
				$errors['phone'] = __('Please enter a valid phone number');
			}
		}

		if ($data['email'] && !is_email($data['email'])) {
			$errors['email'] = __('Not valid Email');
		}

		if ($data['full_name']) {
			if (strlen($data['full_name']) < 3){
				$errors['full_name'] = __('Full Name must be greater than 3 characters');
			}
		}else{
			$errors['full_name'] = __('Full Name is required.');
		}

		if ( count($errors ) > 0 ) {
			wp_send_json_error( ['errors' => $errors, 'message' => __('Error occurred!', 'bookit')] );
		}
	}

	/**
	 * Save Staff
	 */
	public static function save() {
		check_ajax_referer('bookit_save_item', 'nonce');

		if ( !current_user_can( 'manage_options' ) ) {
			return false;
		}

		$data = CleanHelper::cleanData($_POST, self::getCleanRules());
		self::validate( $data );

		if ( ! empty( $data ) ) {
			$id             = ( ! empty( $data['id'] ) ) ? $data['id'] : null;
			$staff_services = json_decode(stripslashes($data['staff_services']));
			$working_hours  = json_decode(stripslashes($data['working_hours']));

			unset( $data['staff_services'] );
			unset( $data['working_hours'] );

			if ( $id ) {
				Staff::update( $data, [ 'id' => $id ] );

				Staff_Services::delete_where( 'staff_id', $id );

				foreach ( $working_hours as $working_hour ) {
					$update = [
						'id'            => $working_hour->id,
						'staff_id'      => $id,
						'weekday'       => $working_hour->weekday,
						'start_time'    => $working_hour->start_time,
						'end_time'      => $working_hour->end_time,
						'break_from'    => $working_hour->break_from,
						'break_to'      => $working_hour->break_to
					];
					Staff_Working_Hours::update( $update, [ 'id' => $update['id'] ] );
				}
			} else {
				Staff::insert( $data );

				$id = Staff::insert_id();

				foreach ( $working_hours as $working_hour ) {
					$insert = [
						'staff_id'      => $id,
						'weekday'       => $working_hour->weekday,
						'start_time'    => $working_hour->start_time,
						'end_time'      => $working_hour->end_time,
						'break_from'    => $working_hour->break_from,
						'break_to'      => $working_hour->break_to
					];
					Staff_Working_Hours::insert( $insert );
				}
			}

			foreach ( $staff_services as $staff_service ) {
				$insert = [
					'staff_id'      => $id,
					'service_id'    => $staff_service->id,
					'price'         => $staff_service->price
				];
				Staff_Services::insert( $insert );
			}

			do_action( 'bookit_staff_saved', $id );

			wp_send_json_success( [ 'id' => $id, 'message' => __( 'Staff Saved!', 'bookit' )] );
		}

		wp_send_json_error( [ 'message' => __('Error occurred!', 'bookit') ] );
	}

	/** Get Staff Assosiated data by id **/
	public static function get_assosiated_total_data_by_id() {
		check_ajax_referer('bookit_get_staff_assosiated_total_data', 'nonce');

		if ( !current_user_can( 'manage_options' ) ) {
			return false;
		}

		$data = CleanHelper::cleanData($_POST, self::getCleanRules());
		if ( empty( $data['id'] ) ) {
			wp_send_json_error( [ 'message' => __('Error occurred!', 'bookit') ] );
		}

		$services       = Staff::get_staff_total_service($data['id']);
		$appointments   = Appointments::get_total_active_assosiated_appointments( '', $data['id'] );

		$response = ['total' => ['services' => $services, 'appointments' => $appointments]];
		wp_send_json_success( $response );
	}

	/**
	 * Delete the Staff
	 */
	public static function delete() {
		check_ajax_referer('bookit_delete_item', 'nonce');

		if ( !current_user_can( 'manage_options' ) ) {
			return false;
		}

		$data = CleanHelper::cleanData($_GET, self::getCleanRules());

		if ( isset( $data['id'] ) ) {

			Staff::deleteStaff($data['id']);

			do_action( 'bookit_staff_deleted', $data['id'] );

			wp_send_json_success( [ 'message' => __('Staff Deleted!', 'bookit') ] );
		}

		wp_send_json_error( [ 'message' => __('Error occurred!', 'bookit') ] );
	}
}